/*
 Highcharts JS v8.0.4 (2020-03-10)

 (c) 2009-2019 Highsoft AS

 License: www.highcharts.com/license
*/
(function(a){"object"===typeof module&&module.exports?(a["default"]=a,module.exports=a):"function"===typeof define&&define.amd?define("highcharts/themes/avocado",["highcharts"],function(b){a(b);a.Highcharts=b;return a}):a("undefined"!==typeof Highcharts?Highcharts:void 0)})(function(a){function b(a,b,c,d){a.hasOwnProperty(b)||(a[b]=d.apply(null,c))}a=a?a._modules:{};b(a,"themes/avocado.js",[a["parts/Globals.js"]],function(a){a.theme={colors:["#F3E796","#95C471","#35729E","#251735"],colorAxis:{maxColor:"#05426E",
minColor:"#F3E796"},plotOptions:{map:{nullColor:"#FCFEFE"}},navigator:{maskFill:"rgba(170, 205, 170, 0.5)",series:{color:"#95C471",lineColor:"#35729E"}}};a.setOptions(a.theme)});b(a,"masters/themes/avocado.btx.js",[],function(){})});
//# sourceMappingURL=avocado.js.map