/**
 * @license Highcharts JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/accessibility
 * @requires highcharts
 *
 * Accessibility module
 *
 * (c) 2010-2019 Highsoft AS
 * Author: Oystein Moseng
 *
 * License: www.highcharts.com/license
 */
'use strict';

import '../../modules/accessibility/accessibility.js';
