/**
 * @license Highstock JS v8.0.4 (2020-03-10)
 * @module highcharts/indicators/indicators
 * @requires highcharts
 * @requires highcharts/modules/stock
 *
 * Slow Stochastic series type for Highstock
 *
 * (c) 2010-2019 Pawel Fus
 *
 * License: www.highcharts.com/license
 */
'use strict';
import '../../indicators/slow-stochastic.btx.js';
