/**
 * @license Highcharts Gantt JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/treegrid
 * @requires highcharts
 *
 * Tree Grid
 *
 * (c) 2016-2019 Jon Arild Nygard
 *
 * License: www.highcharts.com/license
 */
'use strict';
import '../../parts-gantt/TreeGrid.js';
