/**
 * @license Highstock JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/stock-tools
 * @requires highcharts
 * @requires highcharts/modules/stock
 *
 * Advanced Highstock tools
 *
 * (c) 2010-2019 Highsoft AS
 * Author: Torstein Honsi
 *
 * License: www.highcharts.com/license
 */
'use strict';
import '../../modules/stock-tools-bindings.js';
import '../../modules/stock-tools-gui.js';
