/*
 Highcharts JS v8.0.4 (2020-03-10)

 (c) 2009-2019 Highsoft AS

 License: www.highcharts.com/license
*/
(function(a){"object"===typeof module&&module.exports?(a["default"]=a,module.exports=a):"function"===typeof define&&define.amd?define("highcharts/themes/high-contrast-dark",["highcharts"],function(b){a(b);a.Highcharts=b;return a}):a("undefined"!==typeof Highcharts?Highcharts:void 0)})(function(a){function b(a,b,c,d){a.hasOwnProperty(b)||(a[b]=d.apply(null,c))}a=a?a._modules:{};b(a,"themes/high-contrast-dark.js",[a["parts/Globals.js"]],function(a){a.theme={colors:"#a6f0ff #70d49e #e898a5 #007faa #f9db72 #f45b5b #1e824c #e7934c #dadfe1 #a0618b".split(" "),
chart:{backgroundColor:"#1f1f20",plotBorderColor:"#606063"},title:{style:{color:"#F0F0F3"}},subtitle:{style:{color:"#F0F0F3"}},xAxis:{gridLineColor:"#707073",labels:{style:{color:"#F0F0F3"}},lineColor:"#707073",minorGridLineColor:"#505053",tickColor:"#707073",title:{style:{color:"#F0F0F3"}}},yAxis:{gridLineColor:"#707073",labels:{style:{color:"#F0F0F3"}},lineColor:"#707073",minorGridLineColor:"#505053",tickColor:"#707073",title:{style:{color:"#F0F0F3"}}},tooltip:{backgroundColor:"rgba(0, 0, 0, 0.85)",
style:{color:"#F0F0F3"}},plotOptions:{series:{dataLabels:{color:"#F0F0F3"},marker:{lineColor:"#333"}},boxplot:{fillColor:"#505053"},candlestick:{lineColor:"white"},errorbar:{color:"white"},map:{nullColor:"#353535"}},legend:{backgroundColor:"transparent",itemStyle:{color:"#F0F0F3"},itemHoverStyle:{color:"#FFF"},itemHiddenStyle:{color:"#606063"},title:{style:{color:"#D0D0D0"}}},credits:{style:{color:"#F0F0F3"}},labels:{style:{color:"#707073"}},drilldown:{activeAxisLabelStyle:{color:"#F0F0F3"},activeDataLabelStyle:{color:"#F0F0F3"}},
navigation:{buttonOptions:{symbolStroke:"#DDDDDD",theme:{fill:"#505053"}}},rangeSelector:{buttonTheme:{fill:"#505053",stroke:"#000000",style:{color:"#eee"},states:{hover:{fill:"#707073",stroke:"#000000",style:{color:"#F0F0F3"}},select:{fill:"#303030",stroke:"#101010",style:{color:"#F0F0F3"}}}},inputBoxBorderColor:"#505053",inputStyle:{backgroundColor:"#333",color:"#F0F0F3"},labelStyle:{color:"#F0F0F3"}},navigator:{handles:{backgroundColor:"#666",borderColor:"#AAA"},outlineColor:"#CCC",maskFill:"rgba(180,180,255,0.2)",
series:{color:"#7798BF",lineColor:"#A6C7ED"},xAxis:{gridLineColor:"#505053"}},scrollbar:{barBackgroundColor:"#808083",barBorderColor:"#808083",buttonArrowColor:"#CCC",buttonBackgroundColor:"#606063",buttonBorderColor:"#606063",rifleColor:"#FFF",trackBackgroundColor:"#404043",trackBorderColor:"#404043"}};a.setOptions(a.theme)});b(a,"masters/themes/high-contrast-dark.btx.js",[],function(){})});
//# sourceMappingURL=high-contrast-dark.js.map