/**
 * @license Highstock JS v8.0.4 (2020-03-10)
 * @module highcharts/indicators/indicators-all
 * @requires highcharts
 * @requires highcharts/modules/stock
 *
 * All technical indicators for Highstock
 *
 * (c) 2010-2019 Pawel Fus
 *
 * License: www.highcharts.com/license
 */
'use strict';

import '../../indicators/indicators.btx.js';
import '../../indicators/accumulation-distribution.btx.js';
import '../../indicators/ao.btx.js';
import '../../indicators/aroon.btx.js';
import '../../indicators/aroon-oscillator.btx.js';
import '../../indicators/atr.btx.js';
import '../../indicators/bollinger-bands.btx.js';
import '../../indicators/cci.btx.js';
import '../../indicators/cmf.btx.js';
import '../../indicators/dpo.btx.js';
import '../../indicators/ema.btx.js';
import '../../indicators/chaikin.btx.js';
import '../../indicators/dema.btx.js';
import '../../indicators/tema.btx.js';
import '../../indicators/trix.btx.js';
import '../../indicators/apo.btx.js';
import '../../indicators/ichimoku-kinko-hyo.btx.js';
import '../../indicators/keltner-channels.btx.js';
import '../../indicators/macd.btx.js';
import '../../indicators/mfi.btx.js';
import '../../indicators/momentum.btx.js';
import '../../indicators/natr.btx.js';
import '../../indicators/pivot-points.btx.js';
import '../../indicators/ppo.btx.js';
import '../../indicators/price-channel.btx.js';
import '../../indicators/price-envelopes.btx.js';
import '../../indicators/psar.btx.js';
import '../../indicators/roc.btx.js';
import '../../indicators/rsi.btx.js';
import '../../indicators/stochastic.btx.js';
import '../../indicators/slow-stochastic.btx.js';
import '../../indicators/supertrend.btx.js';
import '../../indicators/volume-by-price.btx.js';
import '../../indicators/vwap.btx.js';
import '../../indicators/williams-r.btx.js';
import '../../indicators/wma.btx.js';
import '../../indicators/zigzag.btx.js';
import '../../indicators/regressions.btx.js';
import '../../indicators/acceleration-bands.btx.js';
import '../../indicators/trendline.btx.js';
