/**
 * @license Highstock JS v8.0.4 (2020-03-10)
 * @module highcharts/highstock
 *
 * (c) 2009-2018 Torstein Honsi
 *
 * License: www.highcharts.com/license
 */
'use strict';
import Highcharts from './highcharts.btx.js';
import './modules/stock.btx.js';

Highcharts.product = 'Highstock';
export default Highcharts;
