/**
 * @license Highcharts JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/histogram-bellcurve
 * @requires highcharts
 *
 * (c) 2010-2019 Highsoft AS
 * Author: Sebastian Domas
 *
 * License: www.highcharts.com/license
 */
'use strict';
import '../../modules/histogram.btx.js';
import '../../modules/bellcurve.btx.js';
