/**
 * @license Highstock JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/stock
 * @requires highcharts
 *
 * Highstock as a plugin for Highcharts
 *
 * (c) 2010-2019 Torstein Honsi
 *
 * License: www.highcharts.com/license
 */
'use strict';
import '../../parts/OrdinalAxis.js';
import './broken-axis.btx.js';
import '../../parts/DataGrouping.js';
import '../../parts/OHLCSeries.js';
import '../../parts/CandlestickSeries.js';
import '../../parts/FlagsSeries.js';
import '../../parts/Scrollbar.js';
import '../../parts/Navigator.js';
import '../../parts/RangeSelector.js';
import '../../parts/StockChart.js';
