/**
 * @license Highcharts JS v8.0.4 (2020-03-10)
 * @module highcharts/highcharts-more
 * @requires highcharts
 *
 * (c) 2009-2018 Torstein Honsi
 *
 * License: www.highcharts.com/license
 */
'use strict';
import '../parts-more/Pane.js';
import '../parts-more/RadialAxis.js';
import '../parts-more/AreaRangeSeries.js';
import '../parts-more/AreaSplineRangeSeries.js';
import '../parts-more/ColumnRangeSeries.js';
import '../parts-more/ColumnPyramidSeries.js';
import '../parts-more/GaugeSeries.js';
import '../parts-more/BoxPlotSeries.js';
import '../parts-more/ErrorBarSeries.js';
import '../parts-more/WaterfallSeries.js';
import '../parts-more/PolygonSeries.js';
import '../parts-more/BubbleSeries.js';
import '../parts-more/PackedBubbleSeries.js';
import '../parts-more/Polar.js';
