/**
 * @license Highcharts JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/oldie
 * @requires highcharts
 *
 * Old IE (v6, v7, v8) module for Highcharts v6+.
 *
 * (c) 2010-2019 Highsoft AS
 * Author: Torstein Honsi
 *
 * License: www.highcharts.com/license
 */
'use strict';
import '../../modules/oldie.btx.js';
