/**
 * @license Highcharts JS v8.0.4 (2020-03-10)
 * @module highcharts/highcharts-3d
 * @requires highcharts
 *
 * 3D features for Highcharts JS
 *
 * License: www.highcharts.com/license
 */
'use strict';
import '../parts-3d/Math.js';
import '../parts-3d/SVGRenderer.js';
import '../parts-3d/Chart.js';
import '../parts-3d/Axis.js';
import '../parts-3d/Series.js';
import '../parts-3d/Column.js';
import '../parts-3d/Pie.js';
import '../parts-3d/Scatter.js';
import '../parts-3d/VMLRenderer.js';
