/**
 * @license Highcharts JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/sonification
 * @requires highcharts
 *
 * Sonification module
 *
 * (c) 2012-2019 Øystein Moseng
 *
 * License: www.highcharts.com/license
 */

'use strict';

import '../../modules/sonification/sonification.js';
