/**
 * @license Highcharts JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/color-axis
 * @requires highcharts
 *
 * ColorAxis module
 *
 * (c) 2012-2019 Pawel Potaczek
 *
 * License: www.highcharts.com/license
 */
'use strict';
import '../../parts-map/ColorAxis.js';
