/**
 * @license Highcharts JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/drilldown
 * @requires highcharts
 *
 * Highcharts Drilldown module
 *
 * Author: Torstein Honsi
 * License: www.highcharts.com/license
 *
 */
'use strict';
import '../../modules/drilldown.btx.js';
