/**
 * @license Highcharts JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/annotations-advanced
 * @requires highcharts
 *
 * Annotations module
 *
 * (c) 2009-2019 Torstein Honsi
 *
 * License: www.highcharts.com/license
 */
'use strict';
import '../../annotations/annotations.btx.js';
import '../../annotations/types/CrookedLine.js';
import '../../annotations/types/ElliottWave.js';
import '../../annotations/types/Tunnel.js';
import '../../annotations/types/InfinityLine.js';
import '../../annotations/types/Fibonacci.js';
import '../../annotations/types/Pitchfork.js';
import '../../annotations/types/VerticalLine.js';
import '../../annotations/types/Measure.js';
import '../../annotations/navigationBindings.js';
import '../../annotations/popup.js';
