/**
 * @license Highstock JS v8.0.4 (2020-03-10)
 * @module highcharts/modules/datagrouping
 * @requires highcharts
 *
 * Data grouping module
 *
 * (c) 2010-2019 Torstein Hønsi
 *
 * License: www.highcharts.com/license
 */
'use strict';
import dataGrouping from '../../parts/DataGrouping.js';
export default dataGrouping;
