$(function() {
  $('#container').highcharts({
  	chart: {
            backgroundColor: '#FCF9F9',
            type: 'line'
     },
    title: {
    //Titulo
      text: 'Temperatura promedio Mensual',
      x: -20 //center
    },
    //Subtitulo
    subtitle: {
      text: 'Fuente: WorldClimate.com',
      x: -20
    },
    xAxis: {
    //Categorias en el eje x
      categories: ['Ene', 'Feb', 'Mar', 'Abr', 'May', 'Jun',
        'Jul', 'Ago', 'Sep', 'Oct', 'Nov', 'Dic'
      ]
    },
    yAxis: {
    //Categorias en el eje y
      title: {
      //Titulo Categorias en el eje x
        text: 'Temperature (°C)'
      },
      plotLines: [{
      //linea base
        value: 0,
        width: 1,
        color: '#808080'
      }]
    },
    tooltip: {
    //herramienta de información
      valueSuffix: '°c'
    },
    credits: {
    //quitando los creditos
    	enabled: false
    },
    legend: {
    //legenda
      layout: 'vertical',
      align: 'right',
      verticalAlign: 'middle',
      borderWidth: 0
    },
    exporting: {
    //deshabilitando el boton de print
      enabled: false
     },
    //datos de serie
    series: [{
      name: 'Tokyo',
      data: [7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6], color: '#EE2222'
    }, {
      name: 'New York',
      data: [-0.2, 0.8, 5.7, 11.3, 17.0, 22.0, 24.8, 24.1, 20.1, 14.1, 8.6, 2.5], color: '#FFD166'
    }, {
      name: 'Berlin',
      data: [-0.9, 0.6, 3.5, 8.4, 13.5, 17.0, 18.6, 17.9, 14.3, 9.0, 3.9, 1.0], color: '#06D6A0'
    }, {
      name: 'London',
      data: [3.9, 4.2, 5.7, 8.5, 11.9, 15.2, 17.0, 16.6, 14.2, 10.3, 6.6, 4.8], color: '#861388'
    }]
  });
});
